% Impose Wegsheider conditions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%   Thermodynamically Consistent Bayesian Analysis    %%%%%%%%%%%
%%%%%%%%%%%       of Closed Biochemical Reaction Systems        %%%%%%%%%%%
%%%%%%%%%%%            Posterior Mode Rate Estimation           %%%%%%%%%%%
%%%%%%%%%%%         Authors: Jenkinson, Zhong, Goutsias         %%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 11/04/2010              %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% If kappa is of length 2M, we assume the ordering is of the original 
% system. We then ignore the dependent variables and reset them 
% according to the free variables. If kappa is of length (M+M1), then
% it is taken to be the free variables (in proper ordering), we generate 
% the dependent variables, and form the full kappa vector in original
% system ordering. The free variables are also returned, so if a full 
% kappa is passed the free kappa is computed.
%

function [logKappaFeas,logKappaFree] = imposeWcond(logKappa,W,permMatM)

%
% Set parameters
%

M  = size(permMatM,1);
M2 = size(W,1);
M1 = M-M2;

%
% Find free variables
%

if length(logKappa)<2*M  % the passed variable is the free variable
    
    logKappaFree      = logKappa;
    newLogKapForward  = logKappa(1:M);
    logKappaForward   = newLogKapForward'*permMatM';
    logKappaForward   = logKappaForward';
    newLogKapBackward = [logKappa((M+1):end);zeros(M2,1)];
    
else % the passed variable is the full kappa in original system's ordering
    
    logKappaForward   = logKappa(1:2:(2*M-1));
    logKappaBackward  = logKappa(2:2:(2*M));
    newLogKapForward  = logKappaForward'*permMatM;
    newLogKapForward  = newLogKapForward';
    newLogKapBackward = logKappaBackward'*permMatM;
    newLogKapBackward = newLogKapBackward';
    logKappaFree      = [newLogKapForward; newLogKapBackward(1:M1)];
    
end

%
% Find depenedent variables according to Wegscheider conditions
%

logKappaDepend = W*logKappaFree;

%
% Find feasible parameters in original ordering
%

newLogKapBackward = [newLogKapBackward(1:M1);logKappaDepend];
logKappaBackward  = newLogKapBackward'*permMatM';
logKappaBackward  = logKappaBackward';

logKappaFeas = zeros(2*M,1);
for m = 1:M
    logKappaFeas(2*m-1) = logKappaForward(m);
    logKappaFeas(2*m)   = logKappaBackward(m);
end