% Posterior mean estimator
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%   Thermodynamically Consistent Bayesian Analysis    %%%%%%%%%%%
%%%%%%%%%%%       of Closed Biochemical Reaction Systems        %%%%%%%%%%%
%%%%%%%%%%%            Posterior Mode Rate Estimation           %%%%%%%%%%%
%%%%%%%%%%%         Authors: Jenkinson, Zhong, Goutsias         %%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 11/04/2010              %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function [pm,mode_MA,obj_MA,internalKappa,ER] = ...
    pmean(kappamode,M,N,L,stoich,logkappa_0,tau,lambda,a,b,y,yp,x0,x0p,...
          pert,tq,tol_high,svdtol,timings,W,permMatM)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Q  = size(y,1)-1;           % number of time points
P  = sum(pert(1:N));        % number of perturbations
xi = 0.1;                   % 1/2 the size of the uniform proposal 
                            % distrtibution hypercube
                                
%
% Calculate singular value decomposition of matrix Xi
%

Xi      = stoich'*stoich;
[U,S,U] = svd(Xi);
S0      = diag(S(S(1:M,1:M) > svdtol*max(max(S))));
M0      = M - size(S0,1);
U0      = U(1:M,1:M-M0);
Xi      = U0*diag(1./diag(S0))*U0';

%
% Calculate ytilde
%

yst = repmat(0,[P+1,N]);
yst(1,:) = y(Q+1,:);            % steady-state data from reference system

if P > 0
    for p = 1:P
        yst(p+1,:) = yp(P*Q+p,:);   % steady-state data from perturbed systems
    end
end
ytilde = sum(yst,1)*stoich./(P+1);
ytilde = ytilde';

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% MA-MCMC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%


[kappaFeas,kappaFree] = imposeWcond(kappamode,W,permMatM);
kappaFreeStart        = kappaFree;
cchat                 = object(y,yp,x0,x0p,pert,kappaFree,N,Xi,ytilde,logkappa_0,...
                               lambda,tau,a,b,[tol_high;tol_high],timings,...
                               tq,W,permMatM);         
mode_MAfree  = kappaFree;
obj_MA       = cchat;
oldKappaProb = exp(cchat);

internalKappa = zeros(length(kappaFree),L);
wtbar = waitbar(0,sprintf('MA-MCMC calculations. Please wait...'));

%
% Begin iterations
%

for iter = 1:L
    try
        waitbar((iter-1)/L,wtbar); 
        epsilon = 2*rand(size(kappaFree))-1;
        kappaPrime = kappaFree + xi*epsilon;     % generate proposal kappa

        cc = object(y,yp,x0,x0p,pert,kappaPrime,N,Xi,ytilde,logkappa_0,...
                    lambda,tau,a,b,[tol_high;tol_high],timings,tq,W,permMatM);    
        newKappaProb = exp(cc);              
        rho = newKappaProb/oldKappaProb;     % generate probability ratio

        % Check for better mode estimator          

        if cc > obj_MA
            obj_MA = cc;
            mode_MAfree = kappaPrime;
        end

        % Decide for new kappa

        if rho >= rand
            kappaFree    = kappaPrime;
            oldKappaProb = newKappaProb;
        end  

        internalKappa(:,iter) = kappaFree;      % keep result
        
    catch ER % if error occurs, return to starting point
        fprintf('\nNumerical problem in MCMC -- Returning to starting point\n');
        kappaFree = kappaFreeStart;            
        internalKappa(:,iter) = kappaFree; 
    end
end

close(wtbar);

%
% calculate posterior mean
%

pmFree = sum(internalKappa,2)./L;   

[pm,pmFree] = imposeWcond(pmFree,W,permMatM);

[mode_MA,mode_MAfree] = imposeWcond(mode_MAfree,W,permMatM);

dlmwrite('../data/MAintegrationSequence.txt',internalKappa,'\t');